@@ -1,6 +1,6 @@
 /* auxprop.c - auxilliary property support
  * Rob Siemborski
- * $Id: auxprop.c,v 1.10 2003/03/19 18:25:27 rjs3 Exp $
+ * $Id: auxprop.c,v 1.13 2003/10/20 15:19:58 rjs3 Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -162,14 +162,15 @@ int prop_dup(struct propctx *src_ctx, struct propctx **dst_ctx)
     struct propctx *retval = NULL;
     unsigned i;
     int result;
-    size_t total_size = 0, values_size;
+    unsigned total_size = 0;
+    size_t values_size;
     
     if(!src_ctx || !dst_ctx) return SASL_BADPARAM;
 
     /* What is the total allocated size of src_ctx? */
     pool = src_ctx->mem_base;
     while(pool) {
-	total_size += pool->size;
+	total_size += (unsigned) pool->size;
 	pool = pool->next;
     }
 
@@ -248,7 +249,7 @@ int prop_request(struct propctx *ctx, const char **names)
     /* Do we need to add ANY? */
     if(!new_values) return SASL_OK;
 
-    /* We always want atleast on extra to mark the end of the array */
+    /* We always want at least one extra to mark the end of the array */
     total_values = new_values + ctx->used_values + 1;
 
     /* Do we need to increase the size of our propval table? */
@@ -256,7 +257,7 @@ int prop_request(struct propctx *ctx, const char **names)
 	unsigned max_in_pool;
 
 	/* Do we need a larger base pool? */
-	max_in_pool = ctx->mem_base->size / sizeof(struct propval);
+	max_in_pool = (unsigned) (ctx->mem_base->size / sizeof(struct propval));
 	
 	if(total_values <= max_in_pool) {
 	    /* Don't increase the size of the base pool, just use what
@@ -477,14 +478,17 @@ int prop_format(struct propctx *ctx, const char *sep, int seplen,
     unsigned needed, flag = 0;
     struct propval *val;
     
-    if(!ctx || !outbuf) return SASL_BADPARAM;
+    if (!ctx || !outbuf) return SASL_BADPARAM;
 
-    if(!sep) seplen = 0;    
-    if(seplen < 0) seplen = strlen(sep);
+    if (!sep) seplen = 0;    
+    if (seplen < 0) seplen = (int) strlen(sep);
+/* If seplen is negative now we have overflow.
+   But if you have a string longer than 2Gb, you are an idiot anyway */
+    if (seplen < 0) return SASL_BADPARAM;
 
     needed = seplen * (ctx->used_values - 1);
     for(val = ctx->values; val->name; val++) {
-	needed += strlen(val->name);
+	needed += (unsigned) strlen(val->name);
     }
     
     if(!outmax) return (needed + 1); /* Because of unsigned funkiness */
@@ -640,7 +644,7 @@ int prop_set(struct propctx *ctx, const char *name,
 	cur->values[nvalues - 2] = ctx->data_end;
 
 	cur->nvalues++;
-	cur->valsize += (size - 1);
+	cur->valsize += ((unsigned) size - 1);
     } else /* Appending an entry */ {
 	char **tmp;
 	size_t size;
@@ -702,7 +706,7 @@ int prop_set(struct propctx *ctx, const char *name,
 	*tmp = ctx->data_end;
 
 	cur->nvalues++;
-	cur->valsize += (size - 1);
+	cur->valsize += ((unsigned) size - 1);
     }
     
     return SASL_OK;
@@ -907,3 +911,85 @@ void _sasl_auxprop_lookup(sasl_server_params_t *sparams,
 		  "could not find auxprop plugin, was searching for '%s'",
 		  plist ? plist : "[all]");
 }
+
+/* Do the callbacks for auxprop stores */
+int sasl_auxprop_store(sasl_conn_t *conn,
+		       struct propctx *ctx, const char *user)
+{
+    sasl_getopt_t *getopt;
+    int ret, found = 0;
+    void *context;
+    const char *plist = NULL;
+    auxprop_plug_list_t *ptr;
+    sasl_server_params_t *sparams = NULL;
+    unsigned userlen = 0;
+
+    if (ctx) {
+	if (!conn || !user)
+	    return SASL_BADPARAM;
+
+	sparams = ((sasl_server_conn_t *) conn)->sparams;
+	userlen = (unsigned) strlen(user);
+    }
+    
+    if(_sasl_getcallback(NULL, SASL_CB_GETOPT, &getopt, &context) == SASL_OK) {
+	ret = getopt(context, NULL, "auxprop_plugin", &plist, NULL);
+	if(ret != SASL_OK) plist = NULL;
+    }
+
+    ret = SASL_OK;
+    if(!plist) {
+	/* Do store in all plugins */
+	for(ptr = auxprop_head; ptr && ret == SASL_OK; ptr = ptr->next) {
+	    found=1;
+	    if (ptr->plug->auxprop_store)
+		ret = ptr->plug->auxprop_store(ptr->plug->glob_context,
+					       sparams, ctx, user, userlen);
+	}
+    } else {
+	char *pluginlist = NULL, *freeptr = NULL, *thisplugin = NULL;
+
+	if(_sasl_strdup(plist, &pluginlist, NULL) != SASL_OK) return SASL_FAIL;
+	thisplugin = freeptr = pluginlist;
+	
+	/* Do store in all *specified* plugins, in order */
+	while(*thisplugin) {
+	    char *p;
+	    int last=0;
+	    
+	    while(*thisplugin && isspace((int)*thisplugin)) thisplugin++;
+	    if(!(*thisplugin)) break;
+	    
+	    for(p = thisplugin;*p != '\0' && !isspace((int)*p); p++);
+	    if(*p == '\0') last = 1;
+	    else *p='\0';
+	    
+	    for(ptr = auxprop_head; ptr && ret == SASL_OK; ptr = ptr->next) {
+		/* Skip non-matching plugins */
+		if((!ptr->plug->name
+		    || strcasecmp(ptr->plug->name, thisplugin)))
+		    continue;
+
+		found=1;
+		if (ptr->plug->auxprop_store)
+		    ret = ptr->plug->auxprop_store(ptr->plug->glob_context,
+						   sparams, ctx, user, userlen);
+	    }
+
+	    if(last) break;
+
+	    thisplugin = p+1;
+	}
+
+	sasl_FREE(freeptr);
+    }
+
+    if(!found) {
+	_sasl_log(NULL, SASL_LOG_ERR,
+		  "could not find auxprop plugin, was searching for %s",
+		  plist ? plist : "[all]");
+	return SASL_FAIL;
+    }
+
+    return ret;
+}
