@@ -1,9 +1,9 @@
 /* canonusr.c - user canonicalization support
  * Rob Siemborski
- * $Id: canonusr.c,v 1.11 2002/12/05 04:26:20 rjs3 Exp $
+ * $Id: canonusr.c,v 1.14 2003/10/20 15:19:58 rjs3 Exp $
  */
 /* 
- * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -65,7 +65,7 @@ static canonuser_plug_list_t *canonuser_head = NULL;
  *                   null-terminate, and get into the outparams
  *
  *                   (handled by INTERNAL plugin) */
-/* Also does auxprop lookups once username is canonoicalized */
+/* Also does auxprop lookups once username is canonicalized */
 /* a zero ulen or alen indicates that it is strlen(value) */
 int _sasl_canon_user(sasl_conn_t *conn,
                      const char *user, unsigned ulen,
@@ -270,7 +270,7 @@ static int _canonuser_internal(const sasl_utils_t *utils,
     unsigned i;
     char *in_buf, *userin;
     const char *begin_u;
-    unsigned u_apprealm = 0;
+    size_t u_apprealm = 0;
     sasl_server_conn_t *sconn = NULL;
 
     if(!utils || !user) return SASL_BADPARAM;
