@@ -1,7 +1,7 @@
 /* common.c - Functions that are common to server and clinet
  * Rob Siemborski
  * Tim Martin
- * $Id: common.c,v 1.92 2003/04/16 19:36:00 rjs3 Exp $
+ * $Id: common.c,v 1.97 2003/10/20 15:19:58 rjs3 Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -59,13 +59,6 @@
 #include <saslplug.h>
 #include "saslint.h"
 
-#ifdef WIN32
-/* need to handle the fact that errno has been defined as a function
-   in a dll, not an extern int */
-# ifdef errno
-#  undef errno
-# endif /* errno */
-#endif /* WIN32 */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -74,7 +67,7 @@ static int _sasl_getpath(void *context __attribute__((unused)), const char **pat
 
 static const char build_ident[] = "$Build: libsasl " PACKAGE "-" VERSION " $";
 
-/* It turns out to be conveinent to have a shared sasl_utils_t */
+/* It turns out to be convenient to have a shared sasl_utils_t */
 LIBSASL_VAR const sasl_utils_t *sasl_global_utils = NULL;
 
 /* Should be a null-terminated array that lists the available mechanisms */
@@ -136,7 +129,7 @@ int _sasl_strdup(const char *in, char **out, size_t *outlen)
 {
   size_t len = strlen(in);
   if (outlen) *outlen = len;
-  *out=sasl_ALLOC(len + 1);
+  *out=sasl_ALLOC((unsigned) len + 1);
   if (! *out) return SASL_NOMEM;
   strcpy((char *) *out, in);
   return SASL_OK;
@@ -233,7 +226,7 @@ int sasl_encodev(sasl_conn_t *conn,
 	if(result != SASL_OK) INTERROR(conn, result);
        
 	*output = conn->encode_buf->data;
-	*outputlen = conn->encode_buf->curlen;
+	*outputlen = (unsigned) conn->encode_buf->curlen;
 
     } else {
 	result = conn->oparams.encode(conn->context, invec, numiov,
@@ -736,12 +729,12 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 	      ((sasl_client_conn_t *)conn)->cparams->ipremoteport
 		  = conn->ipremoteport;
 	      ((sasl_client_conn_t *)conn)->cparams->ipremlen =
-		  strlen(conn->ipremoteport);
+		  (unsigned) strlen(conn->ipremoteport);
 	  } else if (conn->type == SASL_CONN_SERVER) {
 	      ((sasl_server_conn_t *)conn)->sparams->ipremoteport
 		  = conn->ipremoteport;
 	      ((sasl_server_conn_t *)conn)->sparams->ipremlen =
-		  strlen(conn->ipremoteport);
+		  (unsigned) strlen(conn->ipremoteport);
 	  }
       } else {
 	  if(conn->type == SASL_CONN_CLIENT) {
@@ -777,12 +770,12 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 	      ((sasl_client_conn_t *)conn)->cparams->iplocalport
 		  = conn->iplocalport;
 	      ((sasl_client_conn_t *)conn)->cparams->iploclen
-		  = strlen(conn->iplocalport);
+		  = (unsigned) strlen(conn->iplocalport);
 	  } else if (conn->type == SASL_CONN_SERVER) {
 	      ((sasl_server_conn_t *)conn)->sparams->iplocalport
 		  = conn->iplocalport;
 	      ((sasl_server_conn_t *)conn)->sparams->iploclen
-		  = strlen(conn->iplocalport);
+		  = (unsigned) strlen(conn->iplocalport);
 	  }
       } else {
 	  if(conn->type == SASL_CONN_CLIENT) {
@@ -877,7 +870,7 @@ const char *sasl_errdetail(sasl_conn_t *conn)
     snprintf(leader,128,"SASL(%d): %s: ",
 	     sasl_usererr(conn->error_code), errstr);
     
-    need_len = strlen(leader) + strlen(conn->error_buf) + 12;
+    need_len = (unsigned) (strlen(leader) + strlen(conn->error_buf) + 12);
     _buf_alloc(&conn->errdetail_buf, &conn->errdetail_buf_len, need_len);
 
     snprintf(conn->errdetail_buf, need_len, "%s%s", leader, conn->error_buf);
@@ -920,7 +913,7 @@ static int _sasl_global_getopt(void *context,
   /* look it up in our configuration file */
   *result = sasl_config_getstring(option, NULL);
   if (*result != NULL) {
-      if (len) { *len = strlen(*result); }
+      if (len) { *len = (unsigned) strlen(*result); }
       return SASL_OK;
   }
 
@@ -1208,6 +1201,7 @@ _sasl_log (sasl_conn_t *conn,
   void *log_ctx;
   
   int ival;
+  unsigned int uval;
   char *cval;
   va_list ap; /* varargs thing */
 
@@ -1312,8 +1306,25 @@ _sasl_log (sasl_conn_t *conn,
 		goto done;
 
 	    done=1;
-
 	    break;
+
+	  case 'o':
+	  case 'u':
+	  case 'x':
+	  case 'X':
+	    frmt[frmtpos++]=fmt[pos];
+	    frmt[frmtpos]=0;
+	    uval = va_arg(ap, unsigned int); /* get the next arg */
+
+	    snprintf(tempbuf,20,frmt,uval); /* have snprintf do the work */
+	    /* now add the string */
+	    result = _sasl_add_string(&out, &alloclen, &outlen, tempbuf);
+	    if (result != SASL_OK)
+		goto done;
+
+	    done=1;
+	    break;
+
 	  default: 
 	    frmt[frmtpos++]=fmt[pos]; /* add to the formating */
 	    frmt[frmtpos]=0;	    
@@ -1418,12 +1429,12 @@ _sasl_alloc_utils(sasl_conn_t *conn,
   utils->prop_set=&prop_set;
   utils->prop_setvals=&prop_setvals;
   utils->prop_erase=&prop_erase;
+  utils->auxprop_store=&sasl_auxprop_store;
 #endif
 
   /* Spares */
   utils->spare_fptr = NULL;
-  utils->spare_fptr1 = utils->spare_fptr2 = 
-      utils->spare_fptr3 = NULL;
+  utils->spare_fptr1 = utils->spare_fptr2 = NULL;
   
   return utils;
 }
@@ -1515,7 +1526,7 @@ _sasl_find_verifyfile_callback(const sasl_callback_t *callbacks)
 int _buf_alloc(char **rwbuf, size_t *curlen, size_t newlen) 
 {
     if(!(*rwbuf)) {
-	*rwbuf = sasl_ALLOC(newlen);
+	*rwbuf = sasl_ALLOC((unsigned)newlen);
 	if (*rwbuf == NULL) {
 	    *curlen = 0;
 	    return SASL_NOMEM;
@@ -1527,7 +1538,7 @@ int _buf_alloc(char **rwbuf, size_t *curlen, size_t newlen)
 	while(needed < newlen)
 	    needed *= 2;
 
-	*rwbuf = sasl_REALLOC(*rwbuf, needed);
+	*rwbuf = sasl_REALLOC(*rwbuf, (unsigned)needed);
 	
 	if (*rwbuf == NULL) {
 	    *curlen = 0;
