@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: server.c,v 1.123 2003/04/16 19:36:01 rjs3 Exp $
+ * $Id: server.c,v 1.133 2003/11/14 16:49:07 rjs3 Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -128,8 +128,10 @@ int sasl_setpass(sasl_conn_t *conn,
 {
     int result=SASL_OK, tmpresult;
     sasl_server_conn_t *s_conn = (sasl_server_conn_t *) conn;
+    const char *password_request[] = { SASL_AUX_PASSWORD_PROP, NULL };
     sasl_server_userdb_setpass_t *setpass_cb = NULL;
     void *context = NULL;
+    int tried_setpass = 0;
     mechanism_t *m;
      
     if (!_sasl_server_active || !mechlist) return SASL_NOTINIT;
@@ -142,10 +144,44 @@ int sasl_setpass(sasl_conn_t *conn,
         || ((flags & SASL_SET_CREATE) && (flags & SASL_SET_DISABLE)))
 	PARAMERROR(conn);
 
+    /* Do we want to store SASL_AUX_PASSWORD_PROP (plain text)?  and
+     * Do we have an auxprop backend that can store properties?
+     */
+    if ((flags & SASL_SET_DISABLE || !(flags & SASL_SET_NOPLAIN)) &&
+	sasl_auxprop_store(NULL, NULL, NULL) == SASL_OK) {
+
+	tried_setpass++;
+
+	if (flags & SASL_SET_DISABLE) {
+	    pass = NULL;
+	    passlen = 0;
+	}
+
+	result = prop_request(s_conn->sparams->propctx, password_request);
+	if (result == SASL_OK) {
+	    result = prop_set(s_conn->sparams->propctx, SASL_AUX_PASSWORD_PROP,
+			      pass, passlen);
+	}
+	if (result == SASL_OK) {
+	    result = sasl_auxprop_store(conn, s_conn->sparams->propctx, user);
+	}
+	if (result != SASL_OK) {
+	    _sasl_log(conn, SASL_LOG_ERR,
+		      "setpass failed for %s: %z",
+		      user, result);
+	} else {
+	    _sasl_log(conn, SASL_LOG_NOTE,
+		      "setpass succeeded for %s", user);
+	}
+    }
+
     /* call userdb callback function */
     result = _sasl_getcallback(conn, SASL_CB_SERVER_USERDB_SETPASS,
 			       &setpass_cb, &context);
     if(result == SASL_OK && setpass_cb) {
+
+	tried_setpass++;
+
 	tmpresult = setpass_cb(conn, context, user, pass, passlen,
 			    s_conn->sparams->propctx, flags);
 	if(tmpresult != SASL_OK) {
@@ -166,6 +202,9 @@ int sasl_setpass(sasl_conn_t *conn,
 	    /* can't set pass for this mech */
 	    continue;
 	}
+
+	tried_setpass++;
+
 	tmpresult = m->plug->setpass(m->plug->glob_context,
 				     ((sasl_server_conn_t *)conn)->sparams,
 				     user,
@@ -198,6 +237,13 @@ int sasl_setpass(sasl_conn_t *conn,
 	}
     }
 
+    if (!tried_setpass) {
+	_sasl_log(conn, SASL_LOG_WARN,
+		  "secret not changed for %s: "
+		  "no writable auxprop plugin or setpass callback found",
+		  user);
+    }
+
     RETURN(conn, result);
 }
 
@@ -322,7 +368,7 @@ static int server_done(void) {
   mechanism_t *m;
   mechanism_t *prevm;
 
-  if(!_sasl_server_active)
+  if(_sasl_server_active == 0)
       return SASL_NOTINIT;
   else
       _sasl_server_active--;
@@ -387,9 +433,9 @@ static int load_config(const sasl_callback_t *verifyfile_cb)
   int result;
   const char *path_to_config=NULL;
   const char *c;
-  unsigned path_len;
+  size_t path_len;
   char *config_filename=NULL;
-  int len;
+  size_t len;
   const sasl_callback_t *getpath_cb=NULL;
 
   /* get the path to the plugins; for now the config file will reside there */
@@ -421,14 +467,14 @@ static int load_config(const sasl_callback_t *verifyfile_cb)
   }
 
   /* construct the filename for the config file */
-  config_filename = sasl_ALLOC(len);
+  config_filename = sasl_ALLOC((unsigned)len);
   if (! config_filename) {
       result = SASL_NOMEM;
       goto done;
   }
 
-  snprintf(config_filename, len, "%.*s/%s.conf", path_len, path_to_config, 
-	   global_callbacks.appname);
+  snprintf(config_filename, len, "%.*s%c%s.conf", path_len, path_to_config, 
+	   HIER_DELIMITER, global_callbacks.appname);
 
   /* Ask the application if it's safe to use this file */
   result = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(verifyfile_cb->context,
@@ -711,6 +757,7 @@ _sasl_transition(sasl_conn_t * conn,
     sasl_getopt_t *getopt;
     int result = SASL_OK;
     void *context;
+    unsigned flags = 0;
 
     if (! conn)
 	return SASL_BADPARAM;
@@ -725,14 +772,20 @@ _sasl_transition(sasl_conn_t * conn,
 	if (dotrans == NULL) dotrans = "n";
     }
 
-    if (*dotrans == '1' || *dotrans == 'y' ||
+
+    if (!strcmp(dotrans, "noplain")) flags |= SASL_SET_NOPLAIN;
+
+    if (flags || *dotrans == '1' || *dotrans == 'y' ||
 	(*dotrans == 'o' && dotrans[1] == 'n') || *dotrans == 't') {
 	/* ok, it's on! */
+	_sasl_log(conn, SASL_LOG_NOTE, 
+		  "transitioning user %s to auxprop database",
+		  conn->oparams.authid);
 	result = sasl_setpass(conn,
 			      conn->oparams.authid,
 			      pass,
 			      passlen,
-			      NULL, 0, 0);
+			      NULL, 0, SASL_SET_CREATE | flags);
     }
 
     RETURN(conn,result);
@@ -773,7 +826,7 @@ int sasl_server_new(const char *service,
   sasl_utils_t *utils;
   sasl_getopt_t *getopt;
   void *context;
-  const char *log_level;
+  const char *log_level, *auto_trans;
 
   if (_sasl_server_active==0) return SASL_NOTINIT;
   if (! pconn) return SASL_FAIL;
@@ -819,17 +872,17 @@ int sasl_server_new(const char *service,
   }
 
   serverconn->sparams->service = (*pconn)->service;
-  serverconn->sparams->servicelen = strlen((*pconn)->service);
+  serverconn->sparams->servicelen = (unsigned) strlen((*pconn)->service);
 
   serverconn->sparams->appname = global_callbacks.appname;
-  serverconn->sparams->applen = strlen(global_callbacks.appname);
+  serverconn->sparams->applen = (unsigned) strlen(global_callbacks.appname);
 
   serverconn->sparams->serverFQDN = (*pconn)->serverFQDN;
-  serverconn->sparams->slen = strlen((*pconn)->serverFQDN);
+  serverconn->sparams->slen = (unsigned) strlen((*pconn)->serverFQDN);
 
   if (user_realm) {
       result = _sasl_strdup(user_realm, &serverconn->user_realm, NULL);
-      serverconn->sparams->urlen = strlen(user_realm);
+      serverconn->sparams->urlen = (unsigned) strlen(user_realm);
       serverconn->sparams->user_realm = serverconn->user_realm;
   } else {
       serverconn->user_realm = NULL;
@@ -838,14 +891,23 @@ int sasl_server_new(const char *service,
 
   serverconn->sparams->callbacks = callbacks;
 
-  log_level = NULL;
+  log_level = auto_trans = NULL;
   if(_sasl_getcallback(*pconn, SASL_CB_GETOPT, &getopt, &context) == SASL_OK) {
     getopt(context, NULL, "log_level", &log_level, NULL);
+    getopt(context, NULL, "auto_transition", &auto_trans, NULL);
   }
   serverconn->sparams->log_level = log_level ? atoi(log_level) : SASL_LOG_ERR;
 
   serverconn->sparams->utils = utils;
-  serverconn->sparams->transition = &_sasl_transition;
+
+  if (auto_trans &&
+      (*auto_trans == '1' || *auto_trans == 'y' || *auto_trans == 't' ||
+       (*auto_trans == 'o' && auto_trans[1] == 'n') ||
+       !strcmp(auto_trans, "noplain")) &&
+      sasl_auxprop_store(NULL, NULL, NULL) == SASL_OK) {
+      serverconn->sparams->transition = &_sasl_transition;
+  }
+
   serverconn->sparams->canon_user = &_sasl_canon_user;
   serverconn->sparams->props = serverconn->base.props;
   serverconn->sparams->flags = flags;
@@ -888,7 +950,7 @@ static int mech_permitted(sasl_conn_t *conn,
     /* get the list of allowed mechanisms (default = all) */
     if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context)
             == SASL_OK) {
-	const char *mlist;
+	const char *mlist = NULL;
 
 	getopt(context, NULL, "mech_list", &mlist, NULL);
 
@@ -1043,7 +1105,7 @@ static int do_authorization(sasl_server_conn_t *s_conn)
 		   s_conn->base.oparams.user, s_conn->base.oparams.ulen,
 		   s_conn->base.oparams.authid, s_conn->base.oparams.alen,
 		   s_conn->user_realm,
-		   (s_conn->user_realm ? strlen(s_conn->user_realm) : 0),
+		   (s_conn->user_realm ? (unsigned) strlen(s_conn->user_realm) : 0),
 		   s_conn->sparams->propctx);
 
     RETURN(&s_conn->base, ret);
@@ -1348,7 +1410,7 @@ static unsigned mech_names_len()
   for (listptr = mechlist->mech_list;
        listptr;
        listptr = listptr->next)
-    result += strlen(listptr->plug->mech_name);
+    result += (unsigned) strlen(listptr->plug->mech_name);
 
   return result;
 }
@@ -1369,7 +1431,7 @@ int _sasl_server_listmech(sasl_conn_t *conn,
   int lup;
   mechanism_t *listptr;
   int ret;
-  int resultlen;
+  size_t resultlen;
   int flag;
   const char *mysep;
 
@@ -1437,7 +1499,7 @@ int _sasl_server_listmech(sasl_conn_t *conn,
       strcat(conn->mechlist_buf,suffix);
 
   if (plen!=NULL)
-      *plen=strlen(conn->mechlist_buf);
+      *plen = (unsigned) strlen(conn->mechlist_buf);
 
   *result = conn->mechlist_buf;
 
@@ -1483,31 +1545,34 @@ sasl_string_list_t *_sasl_server_mechs(void)
 #define EOSTR(s,n) (((s)[n] == '\0') || ((s)[n] == ' ') || ((s)[n] == '\t'))
 static int is_mech(const char *t, const char *m)
 {
-    int sl = strlen(m);
+    size_t sl = strlen(m);
     return ((!strncasecmp(m, t, sl)) && EOSTR(t, sl));
 }
 
 /* returns OK if it's valid */
 static int _sasl_checkpass(sasl_conn_t *conn,
 			   const char *user,
-			   unsigned userlen __attribute__((unused)),
+			   unsigned userlen,
 			   const char *pass,
-			   unsigned passlen __attribute__((unused)))
+			   unsigned passlen)
 {
     sasl_server_conn_t *s_conn = (sasl_server_conn_t *) conn;
     int result;
     sasl_getopt_t *getopt;
     sasl_server_userdb_checkpass_t *checkpass_cb;
     void *context;
-    const char *mlist, *mech;
+    const char *mlist = NULL, *mech = NULL;
     struct sasl_verify_password_s *v;
     const char *service = conn->service;
 
+    if (!userlen) userlen = (unsigned) strlen(user);
+    if (!passlen) passlen = (unsigned) strlen(pass);
+
     /* call userdb callback function, if available */
     result = _sasl_getcallback(conn, SASL_CB_SERVER_USERDB_CHECKPASS,
 			       &checkpass_cb, &context);
     if(result == SASL_OK && checkpass_cb) {
-	result = checkpass_cb(conn, context, user, pass, strlen(pass),
+	result = checkpass_cb(conn, context, user, pass, passlen,
 			      s_conn->sparams->propctx);
 	if(result == SASL_OK)
 	    return SASL_OK;
@@ -1537,6 +1602,9 @@ static int _sasl_checkpass(sasl_conn_t *conn,
 	    while (*mech && !isspace((int) *mech)) mech++;
 	    while (*mech && isspace((int) *mech)) mech++;
 	}
+	else if (!is_mech(mech, "auxprop") && s_conn->sparams->transition) {
+	    s_conn->sparams->transition(conn, pass, passlen);
+	}
     }
 
     if (result == SASL_NOMECH) {
@@ -1565,7 +1633,7 @@ static int _sasl_checkpass(sasl_conn_t *conn,
  */
 int sasl_checkpass(sasl_conn_t *conn,
 		   const char *user,
-		   unsigned userlen __attribute__((unused)),
+		   unsigned userlen,
 		   const char *pass,
 		   unsigned passlen)
 {
@@ -1584,17 +1652,19 @@ int sasl_checkpass(sasl_conn_t *conn,
 	PARAMERROR(conn);
 
     /* canonicalize the username */
-    result = _sasl_canon_user(conn, user, 0,
+    result = _sasl_canon_user(conn, user, userlen,
 			      SASL_CU_AUTHID | SASL_CU_AUTHZID,
 			      &(conn->oparams));
     if(result != SASL_OK) RETURN(conn, result);
     user = conn->oparams.user;
 
     /* Check the password */
-    result = _sasl_checkpass(conn, user, strlen(user), pass, strlen(pass));
+    result = _sasl_checkpass(conn, user, userlen, pass, passlen);
 
-    if (result == SASL_OK)      
-	result = _sasl_transition(conn, pass, passlen);
+    /* Do authorization */
+    if(result == SASL_OK) {
+      result = do_authorization((sasl_server_conn_t *)conn);
+    }
 
     RETURN(conn,result);
 }
@@ -1618,7 +1688,7 @@ int sasl_user_exists(sasl_conn_t *conn,
 		     const char *user) 
 {
     int result=SASL_NOMECH;
-    const char *mlist, *mech;
+    const char *mlist = NULL, *mech = NULL;
     void *context;
     sasl_getopt_t *getopt;
     struct sasl_verify_password_s *v;
