@@ -3,11 +3,11 @@
  * Rob Siemborski
  * Tim Martin
  * split from common.c by Rolf Braun
- * $Id: seterror.c,v 1.6 2002/10/02 14:01:03 rjs3 Exp $
+ * $Id: seterror.c,v 1.8 2003/10/20 15:19:59 rjs3 Exp $
  */
 
 /* 
- * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -105,8 +105,8 @@ void sasl_seterror(sasl_conn_t *conn,
 		   const char *fmt, ...) 
 {
   size_t outlen=0; /* current length of output buffer */
-  int pos=0; /* current position in format string */
-  int formatlen;
+  size_t pos = 0; /* current position in format string */
+  size_t formatlen;
   int result;
   sasl_log_t *log_cb;
   void *log_ctx;
