@@ -1,7 +1,7 @@
 /* GSSAPI SASL plugin
  * Leif Johansson
  * Rob Siemborski (SASL v2 Conversion)
- * $Id: gssapi.c,v 1.74 2003/05/30 23:17:48 rjs3 Exp $
+ * $Id: gssapi.c,v 1.82 2003/11/11 16:26:08 ken3 Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -80,14 +80,9 @@
 
 #include <errno.h>
 
-#ifdef WIN32
-/* This must be after sasl.h */
-# include "saslgssapi.h"
-#endif /* WIN32 */
-
 /*****************************  Common Section  *****************************/
 
-static const char plugin_id[] = "$Id: gssapi.c,v 1.74 2003/05/30 23:17:48 rjs3 Exp $";
+static const char plugin_id[] = "$Id: gssapi.c,v 1.82 2003/11/11 16:26:08 ken3 Exp $";
 
 static const char * GSSAPI_BLANK_STRING = "";
 
@@ -96,14 +91,35 @@ extern gss_OID gss_nt_service_name;
 #define GSS_C_NT_HOSTBASED_SERVICE gss_nt_service_name
 #endif
 
+#ifdef WANT_KERBEROS5_3DES
+/* Check if CyberSafe flag is defined */
+#ifdef CSF_GSS_C_DES3_FLAG
+#define K5_MAX_SSF	112
+#endif
+
+/* Heimdal and MIT use the following */
+#ifdef GSS_KRB5_CONF_C_QOP_DES3_KD
+#define K5_MAX_SSF	112
+#endif
+
+#endif
+
+#ifndef K5_MAX_SSF
+/* All Kerberos implementations support DES */
+#define K5_MAX_SSF	56
+#endif
+
 /* GSSAPI SASL Mechanism by Leif Johansson <leifj@matematik.su.se>
  * inspired by the kerberos mechanism and the gssapi_server and
  * gssapi_client from the heimdal distribution by Assar Westerlund
  * <assar@sics.se> and Johan Danielsson <joda@pdc.kth.se>. 
  * See the configure.in file for details on dependencies.
- * Heimdal can be obtained from http://www.pdc.kth.se/heimdal
  *
  * Important contributions from Sam Hartman <hartmans@fundsxpress.com>.
+ *
+ * This code was tested with the following distributions of Kerberos:
+ * Heimdal (http://www.pdc.kth.se/heimdal), MIT (http://web.mit.edu/kerberos/www/)
+ * CyberSafe (http://www.cybersafe.com/) and SEAM.
  */
 
 typedef struct context {
@@ -118,12 +134,7 @@ typedef struct context {
     const sasl_utils_t *utils;
     
     /* layers buffering */
-    char *buffer;
-    int bufsize;
-    char sizebuf[4];
-    int cursize;
-    int size;
-    unsigned needsize;
+    decode_context_t decode_context;
     
     char *encode_buf;                /* For encoding/decoding mem management */
     char *decode_buf;
@@ -147,8 +158,13 @@ enum {
     SASL_GSSAPI_STATE_AUTHENTICATED = 4
 };
 
+/* sasl_gss_log: only logs status string returned from gss_display_status() */
+#define sasl_gss_log(x,y,z) sasl_gss_seterror_(x,y,z,1)
+#define sasl_gss_seterror(x,y,z) sasl_gss_seterror_(x,y,z,0)
+
 static void
-sasl_gss_seterror(const sasl_utils_t *utils, OM_uint32 maj, OM_uint32 min)
+sasl_gss_seterror_(const sasl_utils_t *utils, OM_uint32 maj, OM_uint32 min,
+	int logonly)
 {
     OM_uint32 maj_stat, min_stat;
     gss_buffer_desc msg;
@@ -172,9 +188,14 @@ sasl_gss_seterror(const sasl_utils_t *utils, OM_uint32 maj, OM_uint32 min)
 				      GSS_C_GSS_CODE, GSS_C_NULL_OID,
 				      &msg_ctx, &msg);
 	if(GSS_ERROR(maj_stat)) {
-	    utils->seterror(utils->conn, 0,
-			    "GSSAPI Failure "
-			    "(could not get major error message)");
+	    if (logonly) {
+		utils->log(utils->conn, SASL_LOG_FAIL,
+			"GSSAPI Failure: (could not get major error message)");
+	    } else {
+		utils->seterror(utils->conn, 0,
+				"GSSAPI Failure "
+				"(could not get major error message)");
+	    }
 	    utils->free(out);
 	    return;
 	}
@@ -212,9 +233,14 @@ sasl_gss_seterror(const sasl_utils_t *utils, OM_uint32 maj, OM_uint32 min)
 				      GSS_C_MECH_CODE, GSS_C_NULL_OID,
 				      &msg_ctx, &msg);
 	if(GSS_ERROR(maj_stat)) {
-	    utils->seterror(utils->conn, 0,
-			    "GSSAPI Failure "
-			    "(could not get minor error message)");
+	    if (logonly) {
+		utils->log(utils->conn, SASL_LOG_FAIL,
+			"GSSAPI Failure: (could not get minor error message)");
+	    } else {
+		utils->seterror(utils->conn, 0,
+				"GSSAPI Failure "
+				"(could not get minor error message)");
+	    }
 	    utils->free(out);
 	    return;
 	}
@@ -244,7 +270,11 @@ sasl_gss_seterror(const sasl_utils_t *utils, OM_uint32 maj, OM_uint32 min)
     
     strcat(out, ")");
     
-    utils->seterror(utils->conn, 0, out);
+    if (logonly) {
+	utils->log(utils->conn, SASL_LOG_FAIL, out);
+    } else {
+	utils->seterror(utils->conn, 0, out);
+    }
     utils->free(out);
 }
 
@@ -340,79 +370,24 @@ static int gssapi_integrity_encode(void *context, const struct iovec *invec,
     return sasl_gss_encode(context,invec,numiov,output,outputlen,0);
 }
 
-#define myMIN(a,b) (((a) < (b)) ? (a) : (b))
-
-static int gssapi_decode_once(void *context,
-			      const char **input, unsigned *inputlen,
-			      char **output, unsigned *outputlen)
+static int gssapi_decode_packet(void *context,
+				const char *input, unsigned inputlen,
+				char **output, unsigned *outputlen)
 {
     context_t *text = (context_t *) context;
     OM_uint32 maj_stat, min_stat;
     gss_buffer_t input_token, output_token;
     gss_buffer_desc real_input_token, real_output_token;
     int result;
-    unsigned diff;
     
     if (text->state != SASL_GSSAPI_STATE_AUTHENTICATED) {
 	SETERROR(text->utils, "GSSAPI Failure");
 	return SASL_NOTDONE;
     }
     
-    /* first we need to extract a packet */
-    if (text->needsize > 0) {
-	/* how long is it? */
-	int tocopy = myMIN(text->needsize, *inputlen);
-	
-	memcpy(text->sizebuf + 4 - text->needsize, *input, tocopy);
-	text->needsize -= tocopy;
-	*input += tocopy;
-	*inputlen -= tocopy;
-	
-	if (text->needsize == 0) {
-	    /* got the entire size */
-	    memcpy(&text->size, text->sizebuf, 4);
-	    text->size = ntohl(text->size);
-	    text->cursize = 0;
-	    
-	    if (text->size > 0xFFFFFF || text->size <= 0) {
-		SETERROR(text->utils, "Illegal size in sasl_gss_decode_once");
-		return SASL_FAIL;
-	    }
-	    
-	    if (text->bufsize < text->size + 5) {
-		result = _plug_buf_alloc(text->utils, &text->buffer,
-					 &(text->bufsize), text->size+5);
-		if(result != SASL_OK) return result;
-	    }
-	}
-	if (*inputlen == 0) {
-	    /* need more data ! */
-	    *outputlen = 0;
-	    *output = NULL;
-	    
-	    return SASL_OK;
-	}
-    }
-    
-    diff = text->size - text->cursize;
-    
-    if (*inputlen < diff) {
-	/* ok, let's queue it up; not enough data */
-	memcpy(text->buffer + text->cursize, *input, *inputlen);
-	text->cursize += *inputlen;
-	*inputlen = 0;
-	*outputlen = 0;
-	*output = NULL;
-	return SASL_OK;
-    } else {
-	memcpy(text->buffer + text->cursize, *input, diff);
-	*input += diff;
-	*inputlen -= diff;
-    }
-    
     input_token = &real_input_token; 
-    real_input_token.value = text->buffer;
-    real_input_token.length = text->size;
+    real_input_token.value = (char *) input;
+    real_input_token.length = inputlen;
     
     output_token = &real_output_token;
     output_token->value = NULL;
@@ -451,10 +426,6 @@ static int gssapi_decode_once(void *context,
 	gss_release_buffer(&min_stat, output_token);
     }
     
-    /* reset for the next packet */
-    text->size = -1;
-    text->needsize = 4;
-    
     return SASL_OK;
 }
 
@@ -465,9 +436,9 @@ static int gssapi_decode(void *context,
     context_t *text = (context_t *) context;
     int ret;
     
-    ret = _plug_decode(text->utils, context, input, inputlen,
+    ret = _plug_decode(&text->decode_context, input, inputlen,
 		       &text->decode_buf, &text->decode_buf_len, outputlen,
-		       gssapi_decode_once);
+		       gssapi_decode_packet, text);
     
     *output = text->decode_buf;
     
@@ -484,8 +455,6 @@ static context_t *gss_new_context(const sasl_utils_t *utils)
     memset(ret,0,sizeof(context_t));
     ret->utils = utils;
     
-    ret->needsize = 4;
-    
     return ret;
 }
 
@@ -540,11 +509,8 @@ static void sasl_gss_free_context_contents(context_t *text)
 	text->utils->free(text->enc_in_buf);
 	text->enc_in_buf = NULL;
     }
-    
-    if (text->buffer) {
-	text->utils->free(text->buffer);
-	text->buffer = NULL;
-    }
+
+    _plug_decode_free(&text->decode_context);
     
     if (text->authid) { /* works for both client and server */
 	text->utils->free(text->authid);
@@ -600,9 +566,10 @@ gssapi_server_mech_step(void *conn_context,
     gss_buffer_t input_token, output_token;
     gss_buffer_desc real_input_token, real_output_token;
     OM_uint32 maj_stat, min_stat;
+    OM_uint32 max_input;
     gss_buffer_desc name_token;
     int ret;
-    
+
     input_token = &real_input_token;
     output_token = &real_output_token;
     output_token->value = NULL; output_token->length = 0;
@@ -683,11 +650,11 @@ gssapi_server_mech_step(void *conn_context,
 				   NULL);
 	
 	if (GSS_ERROR(maj_stat)) {
+	    sasl_gss_log(text->utils, maj_stat, min_stat);
+	    text->utils->seterror(text->utils->conn, SASL_NOLOG, "GSSAPI Failure: gss_accept_sec_context");
 	    if (output_token->value) {
 		gss_release_buffer(&min_stat, output_token);
 	    }
-	    text->utils->seterror(text->utils->conn, SASL_NOLOG, "GSSAPI Failure: gss_accept_sec_context");
-	    text->utils->log(NULL, SASL_LOG_DEBUG, "GSSAPI Failure: gss_accept_sec_context");
 	    sasl_gss_free_context_contents(text);
 	    return SASL_BADAUTH;
 	}
@@ -772,7 +739,13 @@ gssapi_server_mech_step(void *conn_context,
 	    
 	    maj_stat = gss_import_name (&min_stat,
 					&name_without_realm,
+	    /* Solaris 8/9 gss_import_name doesn't accept GSS_C_NULL_OID here,
+	       so use GSS_C_NT_USER_NAME instead if available.  */
+#ifdef HAVE_GSS_C_NT_USER_NAME
 					GSS_C_NT_USER_NAME,
+#else
+					GSS_C_NULL_OID,
+#endif
 					&without);
 	    
 	    if (GSS_ERROR(maj_stat)) {
@@ -866,7 +839,7 @@ gssapi_server_mech_step(void *conn_context,
 	    && params->props.maxbufsize) {
 	    sasldata[0] |= 2;
 	}
-	if (text->requiressf <= 56 && text->limitssf >= 56
+	if (text->requiressf <= K5_MAX_SSF && text->limitssf >= K5_MAX_SSF
 	    && params->props.maxbufsize) {
 	    sasldata[0] |= 4;
 	}
@@ -943,11 +916,12 @@ gssapi_server_mech_step(void *conn_context,
 	    oparams->encode=&gssapi_integrity_encode;
 	    oparams->decode=&gssapi_decode;
 	    oparams->mech_ssf=1;
-	} else if (layerchoice == 4 && text->requiressf <= 56 &&
-		   text->limitssf >= 56) { /* privacy */
+	} else if (layerchoice == 4 && text->requiressf <= K5_MAX_SSF &&
+		   text->limitssf >= K5_MAX_SSF) { /* privacy */
 	    oparams->encode = &gssapi_privacy_encode;
 	    oparams->decode = &gssapi_decode;
-	    oparams->mech_ssf = 56;
+	    /* FIX ME: Need to extract the proper value here */
+	    oparams->mech_ssf = K5_MAX_SSF;
 	} else {
 	    /* not a supported encryption layer */
 	    SETERROR(text->utils,
@@ -1010,14 +984,25 @@ gssapi_server_mech_step(void *conn_context,
             (((unsigned char *) output_token->value)[3] << 0);
 
 	if (oparams->mech_ssf) {
-	    /* xxx this is probably too big */
-	    oparams->maxoutbuf -= 50;
+ 	    maj_stat = gss_wrap_size_limit( &min_stat,
+					    text->gss_ctx,
+					    1,
+					    GSS_C_QOP_DEFAULT,
+					    (OM_uint32) oparams->maxoutbuf,
+					    &max_input);
+	    
+	    oparams->maxoutbuf -= (max_input - oparams->maxoutbuf);
 	}
 	
 	gss_release_buffer(&min_stat, output_token);
 	
 	text->state = SASL_GSSAPI_STATE_AUTHENTICATED;
 	
+	/* used by layers */
+	_plug_decode_init(&text->decode_context, text->utils,
+			  (params->props.maxbufsize > 0xFFFFFF) ? 0xFFFFFF :
+			  params->props.maxbufsize);
+	
 	oparams->doneflag = 1;
 	
 	return SASL_OK;
@@ -1036,7 +1021,7 @@ static sasl_server_plug_t gssapi_server_plugins[] =
 {
     {
 	"GSSAPI",			/* mech_name */
-	56,				/* max_ssf */
+	K5_MAX_SSF,			/* max_ssf */
 	SASL_SEC_NOPLAINTEXT
 	| SASL_SEC_NOACTIVE
 	| SASL_SEC_NOANONYMOUS
@@ -1068,7 +1053,7 @@ int gssapiv2_server_plug_init(
     int *plugcount)
 {
 #ifdef HAVE_GSSKRB5_REGISTER_ACCEPTOR_IDENTITY
-    const char *keytab;
+    const char *keytab = NULL;
     char keytab_path[1024];
     unsigned int rl;
 #endif
@@ -1250,12 +1235,15 @@ static int gssapi_client_mech_step(void *conn_context,
 	if(params->props.max_ssf > params->external_ssf) {
 	    /* We are requesting a security layer */
 	    req_flags |= GSS_C_INTEG_FLAG;
-	    if(params->props.max_ssf - params->external_ssf > 56) {
+	    /* Any SSF bigger than 1 is confidentiality. */
+	    /* Let's check if the client of the API requires confidentiality,
+	       and it wasn't already provided by an external layer */
+	    if(params->props.max_ssf - params->external_ssf > 1) {
 		/* We want to try for privacy */
 		req_flags |= GSS_C_CONF_FLAG;
 	    }
 	}
-	
+
 	maj_stat = gss_init_sec_context(&min_stat,
 					GSS_C_NO_CREDENTIAL,
 					&text->gss_ctx,
@@ -1302,6 +1290,7 @@ static int gssapi_client_mech_step(void *conn_context,
 					   NULL,       /* targ_name */
 					   NULL,       /* lifetime */
 					   NULL,       /* mech */
+					   /* FIX ME: Should check the resulting flags here */
 					   NULL,       /* flags */
 					   NULL,       /* local init */
 					   NULL);      /* open */
@@ -1375,7 +1364,7 @@ static int gssapi_client_mech_step(void *conn_context,
 	}
 	
 	/* taken from kerberos.c */
-	if (secprops->min_ssf > (56 + external)) {
+	if (secprops->min_ssf > (K5_MAX_SSF + external)) {
 	    return SASL_TOOWEAK;
 	} else if (secprops->min_ssf > secprops->max_ssf) {
 	    return SASL_BADPARAM;
@@ -1398,11 +1387,12 @@ static int gssapi_client_mech_step(void *conn_context,
 	serverhas = ((char *)output_token->value)[0];
 	
 	/* if client didn't set use strongest layer available */
-	if (allowed >= 56 && need <= 56 && (serverhas & 4)) {
+	if (allowed >= K5_MAX_SSF && need <= K5_MAX_SSF && (serverhas & 4)) {
 	    /* encryption */
 	    oparams->encode = &gssapi_privacy_encode;
 	    oparams->decode = &gssapi_decode;
-	    oparams->mech_ssf = 56;
+	    /* FIX ME: Need to extract the proper value here */
+	    oparams->mech_ssf = K5_MAX_SSF;
 	    mychoice = 4;
 	} else if (allowed >= 1 && need <= 1 && (serverhas & 2)) {
 	    /* integrity */
@@ -1511,6 +1501,11 @@ static int gssapi_client_mech_step(void *conn_context,
 	
 	oparams->doneflag = 1;
 	
+	/* used by layers */
+	_plug_decode_init(&text->decode_context, text->utils,
+			  (params->props.maxbufsize > 0xFFFFFF) ? 0xFFFFFF :
+			  params->props.maxbufsize);
+	
 	return SASL_OK;
     }
 	
@@ -1531,12 +1526,13 @@ static sasl_client_plug_t gssapi_client_plugins[] =
 {
     {
 	"GSSAPI",			/* mech_name */
-	56,				/* max_ssf */
+	K5_MAX_SSF,			/* max_ssf */
 	SASL_SEC_NOPLAINTEXT
 	| SASL_SEC_NOACTIVE
 	| SASL_SEC_NOANONYMOUS
 	| SASL_SEC_MUTUAL_AUTH,		/* security_flags */
-	SASL_FEAT_WANT_CLIENT_FIRST
+	SASL_FEAT_NEEDSERVERFQDN
+	| SASL_FEAT_WANT_CLIENT_FIRST
 	| SASL_FEAT_ALLOWS_PROXY,	/* features */
 	gssapi_required_prompts,	/* required_prompts */
 	NULL,				/* glob_context */
