@@ -1,7 +1,7 @@
 /* testsuite.c -- Stress the library a little
  * Rob Siemborski
  * Tim Martin
- * $Id: testsuite.c,v 1.33 2003/03/28 19:59:45 rjs3 Exp $
+ * $Id: testsuite.c,v 1.35 2003/08/29 17:06:24 rjs3 Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -75,13 +75,19 @@
 #include <time.h>
 #include <string.h>
 #include <ctype.h>
+#ifndef WIN32
 #include <netinet/in.h>
 #include <netdb.h>
 #include <sys/socket.h>
-#include <sys/file.h>
-#include <netinet/in.h>
-#include <netdb.h>
 #include <arpa/inet.h>
+#include <sys/file.h>
+#endif
+
+#ifdef WIN32
+__declspec(dllimport) char *optarg;
+__declspec(dllimport) int optind;
+__declspec(dllimport) int getsubopt(char **optionp, char * const *tokens, char **valuep);
+#endif
 
 char myhostname[1024+1];
 #define MAX_STEPS 7 /* maximum steps any mechanism takes */
@@ -589,25 +595,27 @@ void test_init(void)
 
     /* sasl_done() before anything */
     sasl_done();
+    if(mem_stat() != SASL_OK) fatal("memory error after sasl_done test");
 
     /* Try passing appname a really long string (just see if it crashes it)*/
 
     result = sasl_server_init(NULL,really_long_string);
     sasl_done();
+    if(mem_stat() != SASL_OK) fatal("memory error after long appname test");
 
     /* try passing NULL name */
     result = sasl_server_init(emptysasl_cb, NULL);
-
     if (result == SASL_OK) fatal("Allowed null name to sasl_server_init");
 
     /* this calls sasl_done when it wasn't inited */
     sasl_done();
+    if(mem_stat() != SASL_OK) fatal("memory error after null appname test");
 
     /* try giving it a different path for where the plugins are */
     result = sasl_server_init(withokpathsasl_cb, "Tester");
-
     if (result!=SASL_OK) fatal("Didn't deal with ok callback path very well");
     sasl_done();
+    if(mem_stat() != SASL_OK) fatal("memory error after callback path test");
 
     /* and the client */
     result = sasl_client_init(withokpathsasl_cb);
@@ -615,12 +623,14 @@ void test_init(void)
     if (result!=SASL_OK)
 	fatal("Client didn't deal with ok callback path very well");
     sasl_done();
+    if(mem_stat() != SASL_OK) fatal("memory error after client test");
 
     /* try giving it an invalid path for where the plugins are */
     result = sasl_server_init(withbadpathsasl_cb, NULL);
 
     if (result==SASL_OK) fatal("Allowed invalid path");
     sasl_done();
+    if(mem_stat() != SASL_OK) fatal("memory error after bad path test");
 
     /* and the client - xxx is this necessary?*/
 #if 0
@@ -2878,6 +2888,17 @@ int main(int argc, char **argv)
     int do_all = 0;
     int skip_do_correct = 0;
     unsigned int seed = time(NULL);
+#ifdef WIN32
+  /* initialize winsock */
+    int result;
+    WSADATA wsaData;
+
+    result = WSAStartup( MAKEWORD(2, 0), &wsaData );
+    if ( result != 0) {
+	fatal("Windows sockets initialization failure");
+    }
+#endif
+
     while ((c = getopt(argc, argv, "Ms:g:r:han")) != EOF)
 	switch (c) {
 	case 'M':
